#! /bin/bash

export SCRIPT_DIR="$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )"
export PROJECT_DIR="$( cd -- "$( dirname -- "$SCRIPT_DIR" )" &> /dev/null && pwd )"
cd $PROJECT_DIR
export PYTHONPATH="$PYTHONPATH:$PROJECT_DIR"
export LIBTPU_INIT_ARGS="--xla_tpu_megacore_fusion_allow_ags=false --xla_enable_async_collective_permute=true --xla_tpu_enable_ag_backward_pipelining=true --xla_tpu_enable_data_parallel_all_reduce_opt=true --xla_tpu_data_parallel_opt_different_sized_ops=true --xla_tpu_enable_async_collective_fusion=true --xla_tpu_enable_async_collective_fusion_multiple_steps=true --xla_tpu_overlap_compute_collective_tc=true --xla_enable_async_all_gather=true"

export llama_tokenizer_path="llava/checkpoints/lwm_checkpoints/tokenizer.model"
export dataset_path="data/cross_attn_nsvq_code8_real_2M_sim_25K_mixed_train.jsonl"
export eval_dataset_path="data/cross_attn_nsvq_code8_real_2M_sim_25K_mixed_val.jsonl"

export output_dir="llava/checkpoints/debug"

export project_id='lwm'
export experiment_note='world-model'
export experiment_id='cross_attn_nsvq_code8_real_2M_sim_25K_mixed_batch128'

# mesh_dim: dp, fsdp, tp, sp
CUDA_VISIBLE_DEVICES=4,5,6,7 python3 -u -m lwm.train \
    --modality='vision,text,delta' \
    --mesh_dim='!-1,4,1,1' \
    --dtype='bf16' \
    --total_steps=13871 \
    --log_freq=1 \
    --delta_tokens=1 \
    --eval_steps=1 \
    --save_model_freq=13870 \
    --eval_log_freq=100 \
    --save_milestone_freq=13870 \
    --load_llama_config='7b' \
    --load_checkpoint="params::/home/World-Model/llava/checkpoints/lwm_checkpoints/params"\
    --update_llama_config="dict(delta_vocab_size=8,theta=50000000,max_sequence_length=2048,use_flash_attention=True,scan_attention=True,scan_query_chunk_size=512,scan_key_chunk_size=1024,remat_attention='nothing_saveable',scan_mlp=True,scan_mlp_chunk_size=8192,remat_mlp='nothing_saveable',remat_block='nothing_saveable',scan_layers=True)" \
    --tokenizer.vocab_file="$llama_tokenizer_path" \
    --optimizer.type='adamw' \
    --llama.delta_vocab_size=8 \
    --optimizer.accumulate_gradient_steps=1 \
    --optimizer.adamw_optimizer.weight_decay=0 \
    --optimizer.adamw_optimizer.lr=2e-5 \
    --optimizer.adamw_optimizer.end_lr=0 \
    --optimizer.adamw_optimizer.lr_warmup_steps=1000 \
    --optimizer.adamw_optimizer.lr_decay_steps=13870 \
    --use_data_sharded_loader=True \
    --train_dataset.type='json_vision_delta' \
    --train_dataset.delta_vision_text_processor.fields_from_example='fields' \
    --train_dataset.delta_vision_text_processor.n_tokens_per_delta=1 \
    --train_dataset.delta_vision_text_processor.max_n_frames=1 \
    --train_dataset.json_delta_dataset.mode="pad" \
    --train_dataset.json_delta_dataset.path="$dataset_path" \
    --train_dataset.json_delta_dataset.seq_length=384 \
    --train_dataset.json_delta_dataset.batch_size=128 \
    --train_dataset.json_delta_dataset.tokenizer_processes=1 \
    --train_dataset.json_delta_dataset.tokenizer_parallel_chunk_size=128 \
    --train_dataset.json_delta_dataset.tokenizer_parallel_batch_size=128 \
    --train_dataset.json_delta_dataset.use_data_sharded_loader=True \
    --eval_dataset.type='json_vision_delta' \
    --eval_dataset.delta_vision_text_processor.fields_from_example='fields' \
    --eval_dataset.delta_vision_text_processor.n_tokens_per_delta=1 \
    --eval_dataset.delta_vision_text_processor.max_n_frames=1 \
    --eval_dataset.json_delta_dataset.mode="pad" \
    --eval_dataset.json_delta_dataset.path="$eval_dataset_path" \
    --eval_dataset.json_delta_dataset.seq_length=384 \
    --eval_dataset.json_delta_dataset.batch_size=128 \
    --eval_dataset.json_delta_dataset.tokenizer_processes=1 \
    --eval_dataset.json_delta_dataset.tokenizer_parallel_chunk_size=128 \
    --eval_dataset.json_delta_dataset.tokenizer_parallel_batch_size=128 \
    --eval_dataset.json_delta_dataset.use_data_sharded_loader=True \
    --checkpointer.save_optimizer_state=False \
    --autoresume=False \
    --logger.append_uuid=False \
    --logger.online=True \
    --logger.project_id="$project_id" \
    --logger.experiment_id="$experiment_id" \
    --logger.experiment_note="$experiment_note" \
    --logger.output_dir="$output_dir" \
    --logger.wandb_dir="$HOME/experiment_output/$project_id"




export SCRIPT_DIR="$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )"
export PROJECT_DIR="$( cd -- "$( dirname -- "$SCRIPT_DIR" )" &> /dev/null && pwd )"
cd $PROJECT_DIR
export PYTHONPATH="$PYTHONPATH:$PROJECT_DIR"
export LIBTPU_INIT_ARGS="--xla_tpu_megacore_fusion_allow_ags=false --xla_enable_async_collective_permute=true --xla_tpu_enable_ag_backward_pipelining=true --xla_tpu_enable_data_parallel_all_reduce_opt=true --xla_tpu_data_parallel_opt_different_sized_ops=true --xla_tpu_enable_async_collective_fusion=true --xla_tpu_enable_async_collective_fusion_multiple_steps=true --xla_tpu_overlap_compute_collective_tc=true --xla_enable_async_all_gather=true"

export llama_tokenizer_path="llava/checkpoints/lwm_checkpoints/tokenizer.model"
export dataset_path="data/lang_table_whole_ver1_processed_1000traj.jsonl"
export eval_dataset_path="data/lang_table_whole_ver1_processed_val.jsonl"

export output_dir="llava/checkpoints/debug"

export project_id='lwm'
export experiment_note='world-model'
export experiment_id='lwm_1000traj_bf16_from_whole_delta8_batch128_real_2M_sim_25K_mixed'

# mesh_dim: dp, fsdp, tp, sp
CUDA_VISIBLE_DEVICES=4,5,6,7 python3 -u -m lwm.train \
    --modality='vision,text,delta' \
    --mesh_dim='!-1,4,1,1' \
    --dtype='bf16' \
    --total_steps=505 \
    --log_freq=1 \
    --delta_tokens=1 \
    --eval_steps=1 \
    --save_model_freq=500 \
    --eval_log_freq=100 \
    --save_milestone_freq=0 \
    --load_llama_config='7b' \
    --load_checkpoint="params::/home/World-Model/llava/checkpoints/debug/cross_attn_nsvq_code8_real_2M_sim_25K_mixed_batch128/streaming_params_13870"\
    --update_llama_config="dict(delta_vocab_size=8,theta=50000000,max_sequence_length=2048,use_flash_attention=True,scan_attention=True,scan_query_chunk_size=512,scan_key_chunk_size=1024,remat_attention='nothing_saveable',scan_mlp=True,scan_mlp_chunk_size=8192,remat_mlp='nothing_saveable',remat_block='nothing_saveable',scan_layers=True)" \
    --tokenizer.vocab_file="$llama_tokenizer_path" \
    --optimizer.type='adamw' \
    --llama.delta_vocab_size=8 \
    --optimizer.accumulate_gradient_steps=1 \
    --optimizer.adamw_optimizer.weight_decay=0.1 \
    --optimizer.adamw_optimizer.lr=2e-5 \
    --optimizer.adamw_optimizer.end_lr=2e-5 \
    --optimizer.adamw_optimizer.lr_warmup_steps=40 \
    --optimizer.adamw_optimizer.lr_decay_steps=505 \
    --use_data_sharded_loader=True \
    --train_dataset.type='json_vision_delta' \
    --train_dataset.delta_vision_text_processor.fields_from_example='fields' \
    --train_dataset.delta_vision_text_processor.n_tokens_per_delta=1 \
    --train_dataset.delta_vision_text_processor.max_n_frames=1 \
    --train_dataset.json_delta_dataset.mode="pad" \
    --train_dataset.json_delta_dataset.path="$dataset_path" \
    --train_dataset.json_delta_dataset.seq_length=384 \
    --train_dataset.json_delta_dataset.batch_size=128 \
    --train_dataset.json_delta_dataset.tokenizer_processes=1 \
    --train_dataset.json_delta_dataset.tokenizer_parallel_chunk_size=128 \
    --train_dataset.json_delta_dataset.tokenizer_parallel_batch_size=128 \
    --train_dataset.json_delta_dataset.use_data_sharded_loader=True \
    --eval_dataset.type='json_vision_delta' \
    --eval_dataset.delta_vision_text_processor.fields_from_example='fields' \
    --eval_dataset.delta_vision_text_processor.n_tokens_per_delta=1 \
    --eval_dataset.delta_vision_text_processor.max_n_frames=1 \
    --eval_dataset.json_delta_dataset.mode="pad" \
    --eval_dataset.json_delta_dataset.path="$eval_dataset_path" \
    --eval_dataset.json_delta_dataset.seq_length=384 \
    --eval_dataset.json_delta_dataset.batch_size=128 \
    --eval_dataset.json_delta_dataset.tokenizer_processes=1 \
    --eval_dataset.json_delta_dataset.tokenizer_parallel_chunk_size=128 \
    --eval_dataset.json_delta_dataset.tokenizer_parallel_batch_size=128 \
    --eval_dataset.json_delta_dataset.use_data_sharded_loader=True \
    --checkpointer.save_optimizer_state=False \
    --autoresume=False \
    --logger.append_uuid=False \
    --logger.online=True \
    --logger.project_id="$project_id" \
    --logger.experiment_id="$experiment_id" \
    --logger.experiment_note="$experiment_note" \
    --logger.output_dir="$output_dir" \
    --logger.wandb_dir="$HOME/experiment_output/$project_id"
